# coding: utf-8

"""
    Usage API

     These usage endpoints define the following operations:  * **Usage**: Retrieve usage data for the specified time period (default is one month).          * Users must have the `View organization usage` permission to access this endpoint.     * This operation offers visibility across all account groups within the organization.     * Users with `View organization usage` permission in multiple organizations should query the operation with the `aid` query string parameter (see optional parameters) for each organization.  * **Quotas**: Obtain organization and account usage quotas. Additionally, users with the appropriate permissions can create, update, or delete these quotas.          * Users must have the necessary permissions to perform quota-related actions.  Refer to the Usage API operations for detailed usage instructions and optional parameters. 

    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import json
from enum import Enum
from typing_extensions import Self


class ExpandUsageOptions(str, Enum):
    """
    ExpandUsageOptions
    """

    """
    allowed enum values
    """
    TEST = 'test'
    ENTERPRISE_MINUS_AGENT = 'enterprise-agent'
    ENTERPRISE_MINUS_AGENT_MINUS_UNIT = 'enterprise-agent-unit'
    ENDPOINT_MINUS_AGENT = 'endpoint-agent'
    ENDPOINT_MINUS_AGENT_MINUS_ESSENTIAL = 'endpoint-agent-essential'
    ENDPOINT_MINUS_AGENT_MINUS_EMBEDDED = 'endpoint-agent-embedded'
    UNKNOWN = 'unknown'

    @classmethod
    def from_json(cls, json_str: str) -> Self:
        """Create an instance of ExpandUsageOptions from a JSON string"""
        return cls(json.loads(json_str))

    @classmethod
    def _missing_(cls, value):
        """Handle unknown values"""
        return cls.UNKNOWN

